
import javax.faces.FacesException;
import javax.el.ELException;
import javax.el.Expression;
import javax.el.ValueExpression;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.faces.context.FacesContext;

import org.ajax4jsf.model.DataVisitor;
import org.ajax4jsf.model.ExtendedDataModel;
import org.ajax4jsf.model.Range;
import org.ajax4jsf.model.SequenceRange;
import org.apache.log4j.Logger;
import org.richfaces.model.ExtendedFilterField;
import org.richfaces.model.ExtendedTableDataModel;
import org.richfaces.model.ExtendedTableDataModifiableModel;
import org.richfaces.model.FilterField;
import org.richfaces.model.Modifiable;
import org.richfaces.model.Ordering;
import org.richfaces.model.SortField2;

import ch.vd.institutions.votelec.controller.datamodel.dataprovider.SortableFilterableDataProvider;
import ch.vd.institutions.votelec.controller.generic.JsfUtils;

/**
 * Highly inspired from richfaces livedemo:  
 * 	- http://anonsvn.jboss.org/repos/richfaces/branches/community/3.3.X/samples/richfaces-demo/src/main/java/org/richfaces/demo/modifiableModel/BaseModifiableHibernateDataModel.java
 */
public class ModifiableExtendedDataModel<T, B extends Enum<?>> extends ExtendedDataModel implements Modifiable{
	private final static Logger LOGGER = Logger.getLogger(ModifiableExtendedDataModel.class);

	private SortableFilterableDataProvider<T, B> dataProvider;

	private Integer rowCount;
	private Object dataItem;
	
	private SequenceRange cachedRange;
	private List<T> cachedItems;

	private List<FilterField> facesFilterFields;
	private List<SortField2> facesSortFields;
	
	public ModifiableExtendedDataModel(SortableFilterableDataProvider<T, B> dataProvider) {
		super();
		this.dataProvider = dataProvider;
	}

	private static boolean areEqualRanges(SequenceRange range1, SequenceRange range2) {
		if (range1 == null || range2 == null) {
			return range1 == null && range2 == null;
		} else {
			return range1.getFirstRow() == range2.getFirstRow() && range1.getRows() == range2.getRows();
		}
	}
	
	private void appendFilters(FacesContext context, List<MyFilterField<B>> filterFields) {
		if (facesFilterFields != null) {
			for (FilterField facesFilterField : facesFilterFields) {
				String columnName = getPropertyName(context, facesFilterField.getExpression());
				
				B column = dataProvider.convertColumnEnum(columnName);

				String filterValue = ((ExtendedFilterField) facesFilterField).getFilterValue();
				
				if (filterValue != null && filterValue.length() != 0) {
					filterFields.add(new MyFilterField<B>(column, filterValue));
				}
			}
		}
	}
	
	private void appendSorts(FacesContext context, List<MySortField<B>> sortFields) {
		if (facesSortFields != null) {
			for (SortField2 facesSortField : facesSortFields) {
				Ordering ordering = facesSortField.getOrdering();
				
				if (Ordering.ASCENDING.equals(ordering) || Ordering.DESCENDING.equals(ordering)) {
					String columnName = getPropertyName(context, facesSortField.getExpression());
					
					B column = dataProvider.convertColumnEnum(columnName);
					
					OrderEnum order = Ordering.ASCENDING.equals(ordering) ? OrderEnum.ASC : OrderEnum.DESC;
					
					sortFields.add(new MySortField<B>(column, order));
				}
			}
		}
	}
	
	private String getPropertyName(FacesContext facesContext, Expression expression) {
		try {
			return (String) ((ValueExpression) expression).getValue(facesContext.getELContext());
		} catch (ELException e) {
			throw new FacesException(e.getMessage(), e);
		}
	}
	
	@Override
	public Object getRowKey() {
		return dataItem;
	}

	@Override
	public void setRowKey(Object key) {
		this.dataItem = key;
	}

	@SuppressWarnings("unchecked")
	@Override
	public void walk(FacesContext facesContext, DataVisitor visitor, Range range, Object argument) throws IOException {

		SequenceRange sequenceRange = (SequenceRange) range;
		
		if (this.cachedItems == null || !areEqualRanges(this.cachedRange, sequenceRange)) {
			List<MySortField<B>> sortFields = new ArrayList<MySortField<B>>();
			List<MyFilterField<B>> filterFields = new ArrayList<MyFilterField<B>>();
			
			appendFilters(facesContext, filterFields);
			appendSorts(facesContext, sortFields);

			// Initialize first and last row to all data
			int firstRow = 0;
			int lastRow = getRowCount();
			
			// If a sequenceRange has been passed as a parameter
			if (sequenceRange != null) {
				// Compute first row
				int newFirstRow = sequenceRange.getFirstRow();
				
				if(newFirstRow > 0){
					firstRow = newFirstRow;
				}
				
				// Compute last row as firstRow plus number of rows
				int numberOfRows = sequenceRange.getRows();
				
				int newLastRow = firstRow + numberOfRows;
				if (newLastRow < lastRow){
					lastRow = newLastRow; 
				}
			}
			
			this.cachedRange = sequenceRange;
			this.cachedItems = dataProvider.getItemsByRange(firstRow, lastRow, filterFields, sortFields);
		}
		
		for (T item: this.cachedItems) {
			visitor.process(facesContext, item, argument);
		}
	}

	// Return total number of rows from provider
	@Override
	public int getRowCount() {
		if (rowCount == null) {
			rowCount = new Integer(dataProvider.getRowCount());
		} 
		
		return rowCount.intValue();
	}

	@Override
	public Object getRowData() {
		return dataItem;
	}

	@Override
	public int getRowIndex() {
		return -1;
	}

	@Override
	public Object getWrappedData() {
		return null;
	}

	@Override
	public boolean isRowAvailable() {
		return (this.dataItem != null);
	}

	@Override
	public void setRowIndex(int rowIndex) {
	}

	@Override
	public void setWrappedData(Object data) {
	}
	
	public void modify(List<FilterField> filterFields, List<SortField2> sortFields) {
		this.facesFilterFields = filterFields;
		this.facesSortFields = sortFields;

		// Reset cached data to reload new data from provider
		// (Don't reload rowCount as this one won't have changed)
		this.cachedItems = null;
		this.cachedRange = null;
	}

}
